package cn.iocoder.yudao.module.system.api.robot;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.module.system.api.robot.dto.RobotCompleteTaskDTO;
import cn.iocoder.yudao.module.system.constant.robot.RobotExecutionStateConstant;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotInformationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskMapper;
import cn.iocoder.yudao.module.system.enums.robot.RobotStatusEnum;
import cn.iocoder.yudao.module.system.enums.robot.RobotTaskDetailStatusEnum;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

@Slf4j
@RestController // 提供 RESTful API 接口，给 Feign 调用
@Validated
public class RobotTaskStatusApiImpl implements RobotTaskStatusApi {

    @Autowired
    private RobotTaskDetailMapper robotTaskDetailMapper;

    @Autowired
    private RobotTaskMapper robotTaskMapper;

    @Autowired@Resource
    private RobotInformationMapper robotInformationMapper;

    /**
     * 机器人完成任务上报
     * @param robotCompleteTaskDTO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public CommonResult<Boolean> robotDoneTask(RobotCompleteTaskDTO robotCompleteTaskDTO) {
        log.info("机器人完成任务上报 :{}", JSON.toJSONString(robotCompleteTaskDTO));
        TenantContextHolder.setTenantId(1L);
        if (RobotExecutionStateConstant.UN_DO.equals(robotCompleteTaskDTO.getExecution_state())) {
            return CommonResult.success(Boolean.TRUE);
        }
        //更新任务状态
        RobotTaskDetailDO detailDO = new RobotTaskDetailDO();
        detailDO.setId(robotCompleteTaskDTO.getOrder_id());
        detailDO.setEndTime(LocalDateTime.now());
        detailDO.setTaskStatus(robotCompleteTaskDTO.getExecution_state());
        robotTaskDetailMapper.updateRobotDetailById(detailDO);
        //更新任务状态
        RobotTaskDetailDO robotTaskDetailDO = robotTaskDetailMapper.selectById(robotCompleteTaskDTO.getOrder_id());
        List<RobotTaskDetailDO> taskDetails = robotTaskDetailMapper.queryByTaskId(robotTaskDetailDO.getRobotTaskId());
        boolean done =
                taskDetails.stream().allMatch(v -> (v.getTaskStatus().equals(RobotTaskDetailStatusEnum.DONE.getType())
                    || v.getTaskStatus().equals(RobotTaskDetailStatusEnum.CLOSE.getType()) ));
        if(done) {
            RobotTaskDO robotTaskDO = new RobotTaskDO();
            robotTaskDO.setId(taskDetails.get(0).getRobotTaskId());
            robotTaskDO.setEndTime(LocalDateTime.now());
            robotTaskDO.setTaskStatus(RobotTaskDetailStatusEnum.DONE.getType());
            robotTaskMapper.updateRobot(robotTaskDO);
        }
        //更新机器人任务状态
        if (RobotExecutionStateConstant.DONE.equals(robotCompleteTaskDTO.getExecution_state()) ||
                RobotExecutionStateConstant.CLOSE.equals(robotCompleteTaskDTO.getExecution_state())) {
            RobotInformationDO query = new RobotInformationDO();
            query.setMacAddress(robotCompleteTaskDTO.getMac());
            List<RobotInformationDO> existRobotMac = robotInformationMapper.queryAllByLimit(query);
            String robotNo = existRobotMac.get(0).getRobotNo();
            robotInformationMapper.updateRobotStatus(robotNo, RobotStatusEnum.STAND_BY.getType());
        }

        return CommonResult.success(true);
    }
}
