package cn.iocoder.yudao.module.system.controller.admin.positionmap;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.dto.NodeBaseDTO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapItemPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapItemRespVO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapItemDO;
import cn.iocoder.yudao.module.system.handler.mapnode.NodeProcessingContext;
import cn.iocoder.yudao.module.system.service.positionmap.PositionMapItemService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 仓库点位地图子表")
@RestController
@RequestMapping("/system/position-map-item")
@Validated
public class PositionMapItemController {

    @Resource
    private PositionMapItemService positionMapItemService;
    @Resource
    private NodeProcessingContext nodeProcessingContext;
    // -- 前端给所有的节点信息 -
    @PostMapping("/batchSaveOrEditOrDel")
    @Operation(summary = "批量新增编辑删除节点")
    public CommonResult<Boolean> batchSaveOrEditOrDel(@RequestParam Long positionMapId, @Valid @RequestBody List<NodeBaseDTO> nodeBaseDTOS) {
        // -- 这里使用策略模式 - 根据各个类型区分 - 如果需要新增类型，新增策略实现类即可 - 如果不需要关联存储具体的表，直接将json存储到dataJson字段即可
        Map<Integer, List<NodeBaseDTO>> map = nodeBaseDTOS.stream().collect(Collectors.groupingBy(NodeBaseDTO::getType));
        // -- 获取到对应地图的所有点位
        List<PositionMapItemDO> oldList = positionMapItemService.getByMapId(positionMapId);
        Map<Integer, List<PositionMapItemDO>> oldMap = oldList.stream().collect(Collectors.groupingBy(PositionMapItemDO::getType));
        for (Map.Entry<Integer, List<NodeBaseDTO>> entry : map.entrySet()) {
            int key = entry.getKey();
            List<PositionMapItemDO> oldItemList = CollectionUtil.isEmpty(oldMap.get(key)) ? Collections.emptyList() : oldMap.get(key);
            nodeProcessingContext.processNodesByStrategy(positionMapId, oldItemList, key, entry.getValue());
        }
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得仓库点位地图子表")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:position-map-item:query')")
    public CommonResult<PositionMapItemRespVO> getPositionMapItem(@RequestParam("id") Long id) {
        PositionMapItemDO positionMapItem = positionMapItemService.getPositionMapItem(id);
        return success(BeanUtils.toBean(positionMapItem, PositionMapItemRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得仓库点位地图子表分页")
    @PreAuthorize("@ss.hasPermission('system:position-map-item:query')")
    public CommonResult<PageResult<PositionMapItemRespVO>> getPositionMapItemPage(@Valid PositionMapItemPageReqVO pageReqVO) {
        PageResult<PositionMapItemDO> pageResult = positionMapItemService.getPositionMapItemPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, PositionMapItemRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出仓库点位地图子表 Excel")
    @PreAuthorize("@ss.hasPermission('system:position-map-item:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportPositionMapItemExcel(@Valid PositionMapItemPageReqVO pageReqVO,
                                           HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<PositionMapItemDO> list = positionMapItemService.getPositionMapItemPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "仓库点位地图子表.xls", "数据", PositionMapItemRespVO.class,
                BeanUtils.toBean(list, PositionMapItemRespVO.class));
    }

}
