package cn.iocoder.yudao.module.system.dal.mysql.robot;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotInformationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotInformationStatisticsVO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 车辆信息 Mapper
 *
 * @author 陈宾顺
 */
@Mapper
public interface RobotInformationMapper extends BaseMapperX<RobotInformationDO> {

    default PageResult<RobotInformationDO> selectPage(RobotInformationPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<RobotInformationDO>()
                .eqIfPresent(RobotInformationDO::getRobotModelId, reqVO.getRobotModelId())
                .eqIfPresent(RobotInformationDO::getRobotModelNumber, reqVO.getRobotModelNumber())
                .eqIfPresent(RobotInformationDO::getRobotNo, reqVO.getRobotNo())
                .eqIfPresent(RobotInformationDO::getRobotTaskModel, reqVO.getRobotTaskModel())
                .eqIfPresent(RobotInformationDO::getMacAddress, reqVO.getMacAddress())
                .eqIfPresent(RobotInformationDO::getUrl, reqVO.getUrl())
                .eqIfPresent(RobotInformationDO::getRobotStatus, reqVO.getRobotStatus())
                .betweenIfPresent(RobotInformationDO::getCreateTime, reqVO.getCreateTime())
                .orderByDesc(RobotInformationDO::getId));
    }

    /**
     * 查询空闲的机器人
     * @return
     */
    List<RobotInformationDO> selectFreeRobot();

    /**
     * 更新机器人状态
     * @param robotNo
     * @param robotStatus
     */
    void updateRobotStatus(@Param("robotNo") String robotNo, @Param("robotStatus") Integer robotStatus);

    /**
     * 查询
     * @param query
     * @return
     */
    List<RobotInformationDO> queryAllByLimit(RobotInformationDO query);

    /**
     * 统计车辆任务
     * @return
     */
    RobotInformationStatisticsVO statisticsInformation();

    /**
     * 根据机器人编号查询
     * @param robotNos
     * @return
     */
    List<RobotInformationDO> selectByRobotNos(@Param("robotNos")  Set<String> robotNos);
}