package cn.iocoder.yudao.module.system.job.robot;

import cn.iocoder.yudao.framework.tenant.core.job.TenantJob;
import cn.iocoder.yudao.module.system.enums.redis.RobotCacheLockEnum;
import cn.iocoder.yudao.module.system.service.robot.job.DistributeTasksService;
import cn.iocoder.yudao.module.system.util.redis.RedissonUtils;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;


@Component
@Slf4j
public class RobotJob {

    @Resource
    private RedissonUtils redissonUtils;

    @Autowired
    private DistributeTasksService distributeTasksService;

    //下发任务给车机
    @XxlJob("DistributeTasksJob")
    @TenantJob
    public void distributeTasksJob() throws InterruptedException {
        log.info("----下发任务给车机----");
        RLock lock = redissonUtils.getLock(RobotCacheLockEnum.ROBOT_TASK_DISTRIBUTE_LOCK.getKey());
        if (lock.tryLock()){
            try {
                distributeTasksService.distributeTasks();
            } catch (Exception e) {
                log.error("下发任务给车机出现异常 :{}",e.getMessage());
            } finally {
                lock.unlock();
            }
        }
    }

}
