package cn.iocoder.yudao.module.system.service.robot;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.module.system.constant.robot.RobotTaskChcheConstant;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotInformationMapper;
import cn.iocoder.yudao.module.system.enums.robot.RobotTaskModelEnum;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 车辆信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RobotInformationServiceImpl implements RobotInformationService {

    @Resource
    private RobotInformationMapper informationMapper;

    @Resource
    private PositionMapMapper positionMapMapper;

    @Resource
    private RedisUtil redisUtil;

    @Override
    public Long createInformation(RobotInformationSaveReqVO createReqVO) {
        //判断mac地址是否重复
        RobotInformationDO query = new RobotInformationDO();
        query.setMacAddress(createReqVO.getMacAddress());
        List<RobotInformationDO> existRobotMac = informationMapper.queryAllByLimit(query);
        if (ObjectUtil.isNotEmpty(existRobotMac)) {
            throw exception(ROBOT_MAC_ADDRESS_EXISTS);
        }
        //判断机器人编号
        query.setMacAddress(null);
        query.setRobotNo(createReqVO.getRobotNo());
        List<RobotInformationDO> existRobotNo = informationMapper.queryAllByLimit(query);
        if (ObjectUtil.isNotEmpty(existRobotNo)) {
            throw exception(ROBOT_ROBOT_NO_EXISTS);
        }
        // 插入
        RobotInformationDO information = BeanUtils.toBean(createReqVO, RobotInformationDO.class);
        informationMapper.insert(information);
        // 返回
        return information.getId();
    }

    @Override
    public void updateInformation(RobotInformationSaveReqVO updateReqVO) {
        // 校验存在
        validateInformationExists(updateReqVO.getId());
        //校验MAC地址
        RobotInformationDO query = new RobotInformationDO();
        query.setMacAddress(updateReqVO.getMacAddress());
        List<RobotInformationDO> existRobot = informationMapper.queryAllByLimit(query);
        boolean exist = existRobot.stream().anyMatch(v -> (v.getMacAddress().equals(updateReqVO.getMacAddress())
                && !v.getId().equals(updateReqVO.getId())));
        if (exist) {
            throw exception(ROBOT_MAC_ADDRESS_EXISTS);
        }
        //校验机器人编号
        query.setMacAddress(null);
        query.setRobotNo(updateReqVO.getRobotNo());
        List<RobotInformationDO> existRobotNo = informationMapper.queryAllByLimit(query);
        boolean robotNoExist = existRobotNo.stream().anyMatch(v -> (v.getRobotNo().equals(updateReqVO.getRobotNo())
                && !v.getId().equals(updateReqVO.getId())));
        if (robotNoExist) {
            throw exception(ROBOT_ROBOT_NO_EXISTS);
        }
        // 更新
        RobotInformationDO updateObj = BeanUtils.toBean(updateReqVO, RobotInformationDO.class);
        informationMapper.updateById(updateObj);
    }

    @Override
    public void deleteInformation(Long id) {
        // 校验存在
        validateInformationExists(id);
        // 删除
        informationMapper.deleteById(id);
    }

    private void validateInformationExists(Long id) {
        if (informationMapper.selectById(id) == null) {
            throw exception(ROBOT_INFORMATION_NOT_EXISTS);
        }
    }

    @Override
    public RobotInformationRespVO getInformation(Long id) {
        RobotInformationDO robotInformationDO = informationMapper.selectById(id);
        RobotInformationRespVO bean = BeanUtils.toBean(robotInformationDO, RobotInformationRespVO.class);
        setPositionMapList(bean);
        return bean;
    }

    public void setPositionMapList(RobotInformationRespVO bean) {
        Set<Long> ids = bean.getFloorAreaJson();
        if (ObjectUtil.isNotEmpty(ids)) {
            List<PositionMapDO> list = positionMapMapper.selectByIds(ids);
            List<RobotPositionMapVO> targetList = BeanUtil.copyToList(list, RobotPositionMapVO.class);
            bean.setPositionMapList(targetList);
        }
    }

    @Override
    public PageResult<RobotInformationRespVO> getInformationPage(RobotInformationPageReqVO pageReqVO) {
        PageResult<RobotInformationDO> pageResult = informationMapper.selectPage(pageReqVO);

        PageResult<RobotInformationRespVO> dataPage = new PageResult<>();
        dataPage.setTotal(pageResult.getTotal());
        List<RobotInformationDO> list = pageResult.getList();
        List<RobotInformationRespVO> targetList = BeanUtil.copyToList(list, RobotInformationRespVO.class);
        targetList.stream().forEach(v -> {
            setPositionMapList(v);
            String electricity = (String)redisUtil.get(RobotTaskChcheConstant.ROBOT_ELECTRICITY + v.getRobotNo());
            v.setElectricity(electricity);
        });
        dataPage.setList(targetList);

        return dataPage;
    }

    /**
     * 统计车辆待命/任务中/离线
     * @return
     */
    @Override
    public RobotInformationStatisticsVO statisticsInformation() {
        RobotInformationStatisticsVO info = informationMapper.statisticsInformation();

        RobotInformationDO query = new RobotInformationDO();
        List<RobotInformationDO> existRobot = informationMapper.queryAllByLimit(query);
        int i = 0;
        if (ObjectUtil.isNotEmpty(existRobot)) {
            for (RobotInformationDO v : existRobot) {
                String electricity = (String)redisUtil.get(RobotTaskChcheConstant.ROBOT_ELECTRICITY + v.getRobotNo());
                if (ObjectUtil.isEmpty(electricity)) {
                    i++;
                }
            }
        }
        info.setOffline(i);
        return info;
    }

    /**
     *
     * @return
     */
    @Override
    public List<RobotInformationDO> getCanUseRobot() {
        RobotInformationDO query = new RobotInformationDO();
        query.setRobotTaskModel(RobotTaskModelEnum.NORMAL.getType());
        return informationMapper.queryAllByLimit(query);
    }

}