package cn.iocoder.yudao.module.system.service.robot;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotModelPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotModelSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotModelDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotModelMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.MODEL_NOT_EXISTS;

/**
 * 车辆类型 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RobotModelServiceImpl implements RobotModelService {

    @Resource
    private RobotModelMapper modelMapper;

    @Override
    public Long createModel(RobotModelSaveReqVO createReqVO) {
        //todo 判断是否存在
//        modelMapper.selectPage(pageReqVO);
        // 插入
        RobotModelDO model = BeanUtils.toBean(createReqVO, RobotModelDO.class);
        modelMapper.insert(model);
        // 返回
        return model.getId();
    }

    @Override
    public void updateModel(RobotModelSaveReqVO updateReqVO) {
        // 校验存在
        validateModelExists(updateReqVO.getId());
        // 更新
        RobotModelDO updateObj = BeanUtils.toBean(updateReqVO, RobotModelDO.class);
        modelMapper.updateById(updateObj);
    }

    @Override
    public void deleteModel(Long id) {
        // 校验存在
        validateModelExists(id);
        // 删除
        modelMapper.deleteById(id);
    }

    private void validateModelExists(Long id) {
        if (modelMapper.selectById(id) == null) {
            throw exception(MODEL_NOT_EXISTS);
        }
    }

    @Override
    public RobotModelDO getModel(Long id) {
        return modelMapper.selectById(id);
    }

    @Override
    public PageResult<RobotModelDO> getModelPage(RobotModelPageReqVO pageReqVO) {
        return modelMapper.selectPage(pageReqVO);
    }

}