package cn.iocoder.yudao.module.system.service.robot.job;

import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.module.mqtt.api.task.RobotTaskApi;
import cn.iocoder.yudao.module.mqtt.api.task.dto.Pose2ds;
import cn.iocoder.yudao.module.mqtt.api.task.dto.RobotAcceptTaskDTO;
import cn.iocoder.yudao.module.mqtt.api.task.dto.RobotAcceptTaskData;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotInformationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskMapper;
import cn.iocoder.yudao.module.system.enums.robot.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class DistributeTasksServiceImpl implements DistributeTasksService {

    @Autowired
    private RobotInformationMapper robotInformationMapper;

    @Autowired
    private RobotTaskMapper robotTaskMapper;

    @Autowired
    private RobotTaskDetailMapper robotTaskDetailMapper;

    @Resource
    private RobotTaskApi robotTaskApi;

    /**
     * 下发搬运任务
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void distributeTasks() {
        TenantContextHolder.setTenantId(1L);
        List<RobotInformationDO> robots = robotInformationMapper.selectFreeRobot();
        if (robots.isEmpty()) {
            log.info("暂无空闲的机器人");
            return;
        }

        List<Long> taskIds = robotTaskMapper.getUnDoAndDoingTaskIds();
        if (ObjectUtil.isEmpty(taskIds)) {
            log.info("暂无需要处理的主任务");
            return;
        }

        List<RobotTaskDetailDO> taskDetailDOS = robotTaskDetailMapper.getUnDoTask(taskIds);
        if (taskDetailDOS.isEmpty()) {
            log.info("暂无需要处理的明细任务");
            return;
        }

        //如果系统暂停  则不下发任务

        //任务下发给机器人
        distributeTasksToRobot(robots,taskDetailDOS);
    }

    /**
     * 任务下发给机器人
     * @param robots
     * @param taskDetailDOS
     */
    @Transactional(rollbackFor = Exception.class)
    public void distributeTasksToRobot(List<RobotInformationDO> robots, List<RobotTaskDetailDO> taskDetailDOS) {
        List<RobotAcceptTaskDTO> robotTaskDOS = new ArrayList<>();
        int i = 0;
        for (RobotTaskDetailDO taskDetailDO : taskDetailDOS) {
            if (robots.size() <= i) {
                break;
            }

            RobotAcceptTaskDTO robotTaskDO = new RobotAcceptTaskDTO();
            robotTaskDO.setOrder_id(taskDetailDO.getId().toString());
            robotTaskDO.setOrder_type("MOVE_OUT");//未定
            robotTaskDO.setTopic(RobotTopicEnum.ROBOT_TASK.getTopic()+robots.get(i).getMacAddress());
            List<RobotAcceptTaskData> data = new ArrayList<>();

            RobotAcceptTaskData taskOne = new RobotAcceptTaskData();
            taskOne.setSerial("1");
            taskOne.setCommand_id(taskDetailDO.getId().toString());
            taskOne.setCommand_type(CommandTypeEnum.MOVE_POSE.getType());

            List<Pose2ds> pose2ds = new ArrayList<>();
            Pose2ds pose2d = new Pose2ds();
            pose2d.setX(-0.700);
            pose2d.setY(-0.678);
            pose2d.setYaw(-0.193);
//            pose2ds.add(pose2d);
            taskOne.setPose2d(pose2d);

           /* RobotAcceptTaskData taskTwo = new RobotAcceptTaskData();
            taskTwo.setSerial("1");
            taskTwo.setCommand_id(taskDetailDO.getId().toString());
            taskTwo.setCommand_type(CommandTypeEnum.FORK.getType());
            taskTwo.setTarget_height("0.05");

            RobotAcceptTaskData taskThree = new RobotAcceptTaskData();
            taskThree.setSerial("2");
            taskThree.setCommand_id(taskDetailDO.getId().toString());
            taskThree.setCommand_type(CommandTypeEnum.FORK.getType());
            taskThree.setTarget_height("0.5");

            RobotAcceptTaskData taskFour = new RobotAcceptTaskData();
            taskFour.setSerial("3");
            taskFour.setCommand_id(taskDetailDO.getId().toString());
            taskFour.setCommand_type(CommandTypeEnum.MOVE_POSE.getType());
            List<Pose2ds> pose2ds4 = new ArrayList<>();
            Pose2ds pose2d4 = new Pose2ds();
            pose2d4.setX(33.02);
            pose2d4.setY(33.02);
            pose2d4.setYaw(3.0);
            pose2ds4.add(pose2d4);
            taskFour.setPose2ds(pose2ds4);

            RobotAcceptTaskData taskFive = new RobotAcceptTaskData();
            taskFive.setSerial("4");
            taskFive.setCommand_id(taskDetailDO.getId().toString());
            taskFive.setCommand_type(CommandTypeEnum.FORK.getType());
            taskFive.setTarget_height("0.05");

            RobotAcceptTaskData taskSix = new RobotAcceptTaskData();
            taskSix.setSerial("5");
            taskSix.setCommand_id(taskDetailDO.getId().toString());
            taskSix.setCommand_type(CommandTypeEnum.FORK.getType());
            List<Pose2ds> pose2ds5 = new ArrayList<>();
            Pose2ds pose2d5 = new Pose2ds();
            pose2d5.setX(55.02);
            pose2d5.setY(55.02);
            pose2d5.setYaw(5.0);
            pose2ds5.add(pose2d5);
            taskSix.setPose2ds(pose2ds5);

            RobotAcceptTaskData taskSeven = new RobotAcceptTaskData();
            taskSeven.setSerial("4");
            taskSeven.setCommand_id(taskDetailDO.getId().toString());
            taskSeven.setCommand_type(CommandTypeEnum.FORK.getType());
            taskSeven.setTarget_height("0.0");*/

            data.add(taskOne);
            /*data.add(taskTwo);
            data.add(taskThree);
            data.add(taskFour);
            data.add(taskFive);
            data.add(taskSix);
            data.add(taskSeven);*/
            robotTaskDO.setData(data);

            robotTaskDOS.add(robotTaskDO);
            //更新任务为处理中
            updateTaskStatus(robots.get(i),taskDetailDO);
            i++;
        }

        robotTaskApi.sendTaskToRobot(robotTaskDOS);
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateTaskStatus(RobotInformationDO robotInformationDO, RobotTaskDetailDO taskDetailDO) {
        robotInformationMapper.updateRobotStatus(robotInformationDO.getRobotNo(), RobotStatusEnum.DOING.getType());
        RobotTaskDetailDO detailDO = new RobotTaskDetailDO();
        detailDO.setId(taskDetailDO.getId());
        detailDO.setStartTime(LocalDateTime.now());
        detailDO.setTaskStatus(RobotTaskDetailStatusEnum.DOING.getType());
        robotTaskDetailMapper.updateRobotDetailById(detailDO);

        RobotTaskDO robotTaskDO = new RobotTaskDO();
        robotTaskDO.setId(taskDetailDO.getRobotTaskId());
        robotTaskDO.setStartTime(LocalDateTime.now());
        robotTaskDO.setTaskStatus(RobotTaskDetailStatusEnum.DOING.getType());
        robotTaskMapper.updateRobot(robotTaskDO);
    }


}
